#include <stdio.h>
#include <string.h>
#include <algorithm>
#include <vector>
#include <bitset>
#include <Windows.h>
#include <assert.h>
#include <stdarg.h>
using namespace std;

#define FOR(i,n) for (int i = 0; i < (int) (n); i++)
#define REP(i, v) for (int i = 0; i < (int) ((v).size()); i++)
#define abs(x) ((x) < 0 ? (-(x)) : (x))

void debugf(const char* format, ...)
{
	va_list arg;
	char buff[1000];
	va_start(arg, format);
	vsnprintf(buff, sizeof(buff), format, arg);
#ifdef VERBOSE
	printf("%s", buff);
#endif
	va_end(arg);
}

typedef long long ll;

const char*  INFILE = "flights.in";
const char* OUTFILE = "flights.out";

struct Ship {
	int spd, startPlanet, cost, cap;
};

struct Flight {
	int sh, eh, mon;
	int people, costPerP;
};

vector<Flight> g[64][64][12]; // from, to, startHour
int allowed[64][64];
int dist[64][64];
int n, m;
Ship ship[64];
DWORD startTicks;
int stats[16];

void readInput()
{
	FILE* f = fopen(INFILE, "rt");
	//
	int xf;
	fscanf(f, "%d%d%d", &n, &m, &xf);
	FOR(i, m) {
		fscanf(f, "%d%d%d%d", &ship[i].startPlanet, &ship[i].spd, &ship[i].cost, &ship[i].cap);
		ship[i].startPlanet--;
	}
	FOR(i, xf) {
		int a, b, d, k;
		fscanf(f, "%d%d%d%d", &a, &b, &d, &k);
		a--, b--;
		allowed[a][b] = 1;
		dist[a][b] = d;
		FOR(j, k) {
			Flight F;
			fscanf(f, "%d%d%d%d%d", &F.sh, &F.eh, &F.mon, &F.costPerP, &F.people);
			F.sh -= 4;
			F.eh -= 4;
			F.mon--;
			if (F.sh >= 0) {
				stats[F.sh]++;
				g[a][b][F.sh].push_back(F);
			}
		}
	}
	//
	fclose(f);
}

struct Itinerary {
	int day, hour, eday, ehour, from, to;
};

struct NextStep {
	int pathLength;
	int next;
	int hinc;
};

struct ShipSchedule {
	bitset<12*50> days[4];
	int length(void) // returns the real length (including the resting day)
	{
		int i = 3;
		while (i >= 0 && !days[i].count()) i--;
		return i + 2;
	}
	vector<Itinerary> getItinerary(int startPlanet, int spd);
};

struct Solution {
	ShipSchedule s[52];
	int lengths[52];
	ll scores[52];
	ll totScore;
	inline bool operator < (const Solution& rhs) const { return totScore < rhs.totScore; }
	bool clash(int h, int d, int p, int len, int maxSh, bool worstCase);
	bool feasible(bool debug = false);
	void calcScores(void);
};


double probHour[16]; // probability of hour to be drawn at random. Sums to 1.
const int LOOKUP_SIZE = 9997;
char rlookup[LOOKUP_SIZE+5];
int occurences[4][6];

void processStatistics(void)
{
	int s = 0;
	FOR(i, 12) s += stats[i];
	FOR(i, 12) {
		probHour[i] = 1.0/12.0;
		if (s) probHour[i] += 4.0 * stats[i] / (double) s;
	}
	double P = 0;
	FOR(i, 12) P += probHour[i];
	FOR(i, 12) {
		probHour[i] /= P;
		assert(probHour[i] > 0);
	}
	int last = 0;
	P = 0;
	FOR(i, 12) {
		P += probHour[i];
		int to = (int) floor(P * LOOKUP_SIZE + 0.5);
		for (int j = last; j < to; j++) {
			rlookup[j] = i;
		}
		last = to;
	}
}

inline int randomHour(void)
{
	return rlookup[rand() % LOOKUP_SIZE];
}

void precalcOccurences(void)
{
	for (int length = 2; length <= 5; length++) {
		for (int doy = 0; doy < 256; doy += length)
			occurences[doy / 64][length]++;
	}
}
NextStep wayback[64][64][4*12+1]; // ship, planet, hoursleft
bool wgood[64][64][4*12+1];

void precalcWayback(void)
{
	memset(wayback, 0, sizeof(wayback));
	memset(wgood, 0, sizeof(wgood));
	FOR(ts, m) FOR(i, 4*12)
		wgood[ts][ship[ts].startPlanet][i] = true;
	FOR(ts, m) {
		FOR(htleft, 4*12) {
			FOR(planet, n) {
				if (!wgood[ts][planet][htleft]) continue;
				int hnow = htleft % 12;
				FOR(fromPlanet, n) if (fromPlanet != planet && allowed[fromPlanet][planet]) {
					int hreq = (dist[fromPlanet][planet] - 1) / ship[ts].spd + 1;
					if (hreq + hnow <= 12 && (!wgood[ts][fromPlanet][htleft + hreq] || 
							wayback[ts][fromPlanet][htleft + hreq].pathLength > wayback[ts][planet][htleft].pathLength + 1)) {
						wgood  [ts][fromPlanet][htleft + hreq] = true;
						wayback[ts][fromPlanet][htleft + hreq].pathLength = wayback[ts][planet][htleft].pathLength + 1;
						wayback[ts][fromPlanet][htleft + hreq].next = planet;
						wayback[ts][fromPlanet][htleft + hreq].hinc = hreq;
					}
				}
				if (!wgood[ts][planet][htleft + 1] || wayback[ts][planet][htleft + 1].pathLength > wayback[ts][planet][htleft].pathLength) {
					wgood  [ts][planet][htleft + 1] = true;
					wayback[ts][planet][htleft + 1].pathLength = wayback[ts][planet][htleft].pathLength;
					wayback[ts][planet][htleft + 1].next = planet;
					wayback[ts][planet][htleft + 1].hinc = 1;
				}
			}
		}
	}
}

vector<Itinerary> lastIt[64];
bool tryNew(Solution& sol, int minHourDiff, bool worstcase)
{
	FOR(ts, m) {
		ShipSchedule& s = sol.s[ts];
		Ship& sh = ship[ts];
		FOR(i, 4) s.days[i].reset();
		int len = rand() % 4 + 1;
		int cday = 0;
		int cplanet = sh.startPlanet;
		int chour = 0;
		vector<Itinerary> flights;
		while (1) {
			int eplanets[64], ne = 0;
			FOR (i, n) if (allowed[cplanet][i]) eplanets[ne++] = i;
			bool ok = false;
			int h, eh;
			int eplanet;
			FOR(i, 10) {
				h = randomHour();
				if (h < chour + minHourDiff) continue;
				eplanet = eplanets[rand() % 1023 % ne];
				int flightHours = (dist[cplanet][eplanet] - 1) / sh.spd + 1;
				eh = h + flightHours;
				if (eh > 12) continue;
				if (sol.clash(h, cday, cplanet, len, ts, worstcase)) continue;
				ok = true;
				break;
			}
			if (ok) {
				Itinerary it = { cday, h, 0, 0, cplanet, eplanet };
				//s.days[cday].set(cplanet*12 + h);
				chour = eh;
				cplanet = eplanet;
				if (chour == 12) {
					chour = 0;
					cday++;
				}
				it.eday = cday;
				it.ehour = chour;
				flights.push_back(it);
			} else {
				chour = 0;
				cday++;
			}
			if (cday == len) break;
		}
		if (flights.empty())
			return false;
		bool foundEnding = false;
		for (int endi = (int) flights.size() - 1; endi >= 0; endi--) {
			int cday = flights[endi].eday;
			int ch = flights[endi].ehour;
			int htleft = 12 - ch + (len - 1 - cday) * 12;
			if (wgood[ts][flights[endi].to][htleft]) {
				bool endingIsGood = false;
				vector<Itinerary> newFlights;
				//flights.erase(flights.begin() + endi + 1, flights.end());
				int cplanet = flights[endi].to;
				while (htleft >= 0 && wayback[ts][cplanet][htleft].pathLength>0) {
					if (!wgood[ts][cplanet][htleft]) break;
					if (wayback[ts][cplanet][htleft].next == cplanet || sol.clash(ch, cday, cplanet, len, ts, worstcase)) {
						htleft--;
						ch++;
						if (ch == 12) {
							ch = 0;
							cday++;
						}
					} else {
						NextStep step = wayback[ts][cplanet][htleft];
						
						Itinerary it = { cday, ch, 0, 0, cplanet, step.next };
						ch += step.hinc;
						htleft -= step.hinc;
						cplanet = step.next;
						assert(ch <= 12);

						if (ch == 12) {
							ch = 0;
							cday++;
						}
						it.eday = cday;
						it.ehour = ch;
						newFlights.push_back(it);
						if (wayback[ts][cplanet][htleft].pathLength == 0) {
							endingIsGood = true;
							break;
						}
					}
				}
				if (endingIsGood) {
					foundEnding = true;
					if (endi != (int) flights.size() - 1) flights.erase(flights.begin() + endi + 1, flights.end());
					REP(i, newFlights) flights.push_back(newFlights[i]);
					break;
				}
			}
		}
		if (foundEnding) {
			int maxDay = 0;
			REP(i, flights) {
				const Itinerary& it = flights[i];
				s.days[it.day].set(it.from * 12 + it.hour);
				maxDay = max(maxDay, it.day);
			}
			sol.lengths[ts] = sol.s[ts].length();
			lastIt[ts] = flights;
		} else {
			return false;
		}
	}
	if (!sol.feasible()) {
		return false;
	}
	sol.calcScores();
	return true;
}

vector<pair<int, int> > daysToCheck_fn(int length1, int length2)
{
	vector<pair<int, int> > v;
	if (length1 == length2) {
		FOR(i, length1-1) v.push_back(make_pair(i, i));
		return v;
	}
	if (length1 == 2 && length2 == 4) {
		v.push_back(make_pair(0, 0));
		v.push_back(make_pair(0, 2));
		return v;
	}
	if (length1 == 4 && length2 == 2) {
		v.push_back(make_pair(0, 0));
		v.push_back(make_pair(2, 0));
		return v;
	}
	FOR(i, length1-1)
		FOR(j, length2 - 1)
			v.push_back(make_pair(i, j));
	return v;
}

vector<pair<int, int> > daysToCheck[6][6];

void precalcDays(void)
{
	for (int l1 = 2; l1 <= 5; l1++)
		for (int l2 = 2; l2 <= 5; l2++)
			daysToCheck[l1][l2] = daysToCheck_fn(l1, l2);
}

bool Solution::clash(int h, int d, int p, int len, int maxSh, bool worstCase)
{
	FOR(ts, maxSh) {
		if (!worstCase) {
			const vector<pair<int, int> >& toCheck = daysToCheck[len][lengths[ts]];
			REP(i, toCheck) {
				if (toCheck[i].first != d) continue;
				if (s[ts].days[toCheck[i].second].test(p * 12 + h)) return true;
			}
		} else {
			FOR(i, lengths[ts] - 1) {
				if (s[ts].days[i].test(p * 12 + h)) return true;
			}
		}
	}
	return false;
}

bool Solution::feasible(bool debug)
{
	FOR(i, m) {
		for (int j = i + 1; j < m; j++) {
			const vector<pair<int, int> >& toCheck = daysToCheck[lengths[i]][lengths[j]];
			REP(k, toCheck) {
				bitset<12*50> res = s[i].days[toCheck[k].first] & s[j].days[toCheck[k].second];
				if (res.any()) {
					if (debug) {
						printf("Clashed, because (%d[%d] vs %d[%d]): \n", i, toCheck[k].first, j, toCheck[k].second);
						FOR(k, m) {
							FOR(d, lengths[k] - 1) {
								printf("Ship %2d, day %d:", k, d);
								FOR(i, n*12)
									printf("%d", s[k].days[d].test(i) ? 1 : 0);
								printf("\n");
							}
							printf("\n");
						}
					}
					return false;
				}
			}
		}
	}
	return true;
}

ll calcProfit(int from, int to, int hour, int endHour, int cap, int length, ll costPerTravel)
{
	const vector<Flight>& v = g[from][to][hour];
	ll res = 0;
	REP(i, v) {
		int passengers = min(cap, v[i].people);
		ll win = 0;
		if (endHour > hour && endHour <= v[i].eh)
			win = (ll) passengers * (ll) v[i].costPerP;
		res += (win - costPerTravel) * (ll) occurences[v[i].mon][length];
	}
	return res;
}

vector<Itinerary> ShipSchedule::getItinerary(int startPlanet, int spd)
{
	vector<pair<int, int> > path;
	for (int ctime = 0; ctime < 4 * 12; ctime++) {
		FOR(pl, n) if (days[ctime/12].test(pl*12 + (ctime%12)))
			path.push_back(make_pair(pl, ctime));
	}
	assert(path[path.size() - 1].first != startPlanet);
	assert(path[0].first == startPlanet);
	path.push_back(make_pair(startPlanet, 4*12));
	vector<Itinerary> v;
	for (int i = 0; i < (int) path.size() - 1; i++) {
		Itinerary it;
		it.from = path[i].first;
		it.to = path[i + 1].first;
		it.day = path[i].second/12;
		it.hour = path[i].second%12;
		int took = (dist[it.from][it.to] - 1) / spd + 1;
		assert(it.hour + took <= 12);
		int ntime = path[i].second + took;
		it.eday = ntime / 12;
		it.ehour = ntime % 12;
		v.push_back(it);
	}
	return v;
}


void Solution::calcScores(void)
{
	this->totScore = 0;
	FOR(ts, m) {
		scores[ts] = 0;
		int cplanet = ship[ts].startPlanet;
		int ctime = 0;
		vector<Itinerary> v = s[ts].getItinerary(cplanet, ship[ts].spd);
		REP(i, v) {
			const Itinerary& it = v[i];
			scores[ts] += calcProfit(it.from, it.to, it.hour, it.ehour, ship[ts].cap, lengths[ts], (ll) dist[it.from][it.to] * (ll)ship[ts].cost);
		}
		totScore += scores[ts];
	}
}

const int TARGETPOP = 500;
const int NMIX = 100;
const int TOTALGENES = 199;
const int FATHERGENES = 133;
const int TIMELIMIT = 1600; //ms
const int NEWBREED = 20;
const int NEWTRIES = 200;
vector<Solution> pop, newpop;

void solve(void)
{
	precalcOccurences();
	processStatistics();
	precalcDays();
	precalcWayback();
	int generation = 0;
#ifdef VERBOSE
	while (GetTickCount() - startTicks < 5000) {
#else
	while (GetTickCount() - startTicks < TIMELIMIT) {
#endif
		generation++;
		newpop.clear();
		int nbirthAttempts = 0;
		while (newpop.size() < NEWBREED && nbirthAttempts++ < NEWTRIES) {
			Solution x;
			if (generation == 1 && nbirthAttempts == 79) 
				debugf("foo!\n");
			if (tryNew(x, rand() % 12, rand() % 199 % 2)) newpop.push_back(x);
		}
		int interbred = 0;
		int newbreeds = (int) newpop.size();
		FOR(tt, NMIX) {
			if (pop.size() <= 1) continue;
			int a = rand() % pop.size();
			int b;
			do {
				b = rand() % pop.size();
			} while (a == b);
			Solution newSol;
			newSol.totScore = 0;
			FOR(i, m) {
				int gene = rand() % TOTALGENES;
				int from = (gene < FATHERGENES) ? a : b;
				newSol.s[i] = pop[from].s[i];
				newSol.lengths[i] = pop[from].lengths[i];
				newSol.scores[i] = pop[from].scores[i];
				newSol.totScore += newSol.scores[i];
			}
			if (newSol.feasible()) {
				newpop.push_back(newSol);
				interbred++;
			}
		}
		REP(i, newpop) pop.push_back(newpop[i]);
		if (pop.size() > TARGETPOP) {
			sort(pop.rbegin(), pop.rend());
			pop.erase(pop.begin() + TARGETPOP, pop.end());
		}
		if (!pop.empty()) {
				debugf("Generation %d (%d): %d newbreeds, %d interbred, max score: %lld\n", generation, (int) pop.size(), newbreeds, interbred, pop[0].totScore);
		} else {
			debugf("--\n");
		}
	}
}

void writeOutput(void)
{
	FILE* f = fopen(OUTFILE, "wt");
	Solution sol = pop[0];
	FOR(i, m) {
		vector<Itinerary> v = sol.s[i].getItinerary(ship[i].startPlanet, ship[i].spd);
		fprintf(f, "%d\n", v.size());
		REP(j, v) {
			fprintf(f, "%d %d %d\n", 1 + v[j].day, 4 + v[j].hour, v[j].to + 1);
		}
	}
	fclose(f);
}

int main(void)
{
	startTicks = GetTickCount();
	srand(42);
	readInput();
	solve();
	writeOutput();
	return 0;
}
